unit adxOLFoldersEvents_IMPL;

interface

uses
  SysUtils, ComObj, ComServ, ActiveX, adxAddIn, adxOLFoldersEvents_TLB, StdVcl,
  {$IFDEF VER130}
  OleServer,
  {$ENDIF VER130}
  Classes, Outlook2000;

type
  TADX_OLFoldersEvents = class(TadxAddin, IADX_OLFoldersEvents)
  end;

  TMyFolders = class;

  TAddInModule = class(TadxCOMAddInModule)
    CmdBar: TadxOlExplorerCommandbar;
    procedure CmdBarControls0Change(Sender: TObject);
    procedure CmdBarControls1Click(Sender: TObject);
    procedure adxCOMAddInModuleAddInStartupComplete(Sender: TObject);
    procedure adxCOMAddInModuleAddInBeginShutdown(Sender: TObject);
  private
    FFolders: TMyFolders;
    procedure ConnectToFolderByIndex(const AIndex: Integer);
  protected
  public
  end;

{$IFDEF VER130}
  TFoldersFolderAdd = procedure(ASender: TObject; const Folder: MAPIFolder) of object;
  TFoldersFolderChange = procedure(ASender: TObject; const Folder: MAPIFolder) of object;

  TFolders = class(TOleServer)
  private
    FOnFolderAdd: TFoldersFolderAdd;
    FOnFolderChange: TFoldersFolderChange;
    FOnFolderRemove: TNotifyEvent;
    FIntf: _Folders;
    function GetDefaultInterface: _Folders;
  protected
    procedure InitServerData; override;
    procedure InvokeEvent(DispID: TDispID; var Params: TVariantArray); override;
    function Get_Application: _Application;
    function Get_Class_: OlObjectClass;
    function Get_Session: _NameSpace;
    function Get_Parent: IDispatch;
    function Get_Count: Integer;
    function Get_RawTable: IUnknown;
  public
    procedure Connect; override;
    procedure ConnectTo(svrIntf: _Folders);
    procedure Disconnect; override;
    function Item(Index: OleVariant): MAPIFolder;
    function Add(const Name: WideString): MAPIFolder; overload;
    function Add(const Name: WideString; Type_: OleVariant): MAPIFolder; overload;
    function GetFirst: MAPIFolder;
    function GetLast: MAPIFolder;
    function GetNext: MAPIFolder;
    function GetPrevious: MAPIFolder;
    procedure Remove(Index: Integer);
    property DefaultInterface: _Folders read GetDefaultInterface;
    property Application: _Application read Get_Application;
    property Class_: OlObjectClass read Get_Class_;
    property Session: _NameSpace read Get_Session;
    property Parent: IDispatch read Get_Parent;
    property Count: Integer read Get_Count;
    property RawTable: IUnknown read Get_RawTable;
  published
    property OnFolderAdd: TFoldersFolderAdd read FOnFolderAdd write FOnFolderAdd;
    property OnFolderChange: TFoldersFolderChange read FOnFolderChange write FOnFolderChange;
    property OnFolderRemove: TNotifyEvent read FOnFolderRemove write FOnFolderRemove;
  end;
{$ENDIF VER130}

  TMyFolders = class(TFolders)
  private
    function GetFolderObj: MAPIFolder;
  protected
    procedure DoFolderAdd(ASender: TObject; const Folder: MAPIFolder);
    procedure DoFolderChange(ASender: TObject; const Folder: MAPIFolder);
    procedure DoFolderRemove(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    procedure ConnectTo(AFolder: MAPIFolder);
    property FolderObj: MAPIFolder read GetFolderObj;
  end;

implementation

{$R *.dfm}

uses Dialogs;

{$IFDEF VER130}

{ TFolders }

procedure TFolders.InitServerData;
const
  CServerData: TServerData = (
    ClassID:   '{00063051-0000-0000-C000-000000000046}';
    IntfIID:   '{00063040-0000-0000-C000-000000000046}';
    EventIID:  '{00063076-0000-0000-C000-000000000046}';
    LicenseKey: nil;
    Version: 500);
begin
  ServerData := @CServerData;
end;

procedure TFolders.Connect;
var
  punk: IUnknown;
begin
  if FIntf = nil then begin
    punk := GetServer;
    ConnectEvents(punk);
    FIntf := punk as _Folders;
  end;
end;

procedure TFolders.ConnectTo(svrIntf: _Folders);
begin
  Disconnect;
  FIntf := svrIntf;
  ConnectEvents(FIntf);
end;

procedure TFolders.DisConnect;
begin
  if Fintf <> nil then begin
    DisconnectEvents(FIntf);
    FIntf := nil;
  end;
end;

function TFolders.GetDefaultInterface: _Folders;
begin
  if FIntf = nil then
    Connect;
  Assert(FIntf <> nil, 'DefaultInterface is NULL. Component is not connected to Server. You must call ''Connect'' or ''ConnectTo'' before this operation');
  Result := FIntf;
end;

procedure TFolders.InvokeEvent(DispID: TDispID; var Params: TVariantArray);
begin
  case DispID of
    -1: Exit;  // DISPID_UNKNOWN
    61441: if Assigned(FOnFolderAdd) then
         FOnFolderAdd(Self, IUnknown(TVarData(Params[0]).VPointer) as MAPIFolder {const MAPIFolder});
    61442: if Assigned(FOnFolderChange) then
         FOnFolderChange(Self, IUnknown(TVarData(Params[0]).VPointer) as MAPIFolder {const MAPIFolder});
    61443: if Assigned(FOnFolderRemove) then
         FOnFolderRemove(Self);
  end; {case DispID}
end;

function TFolders.Get_Application: _Application;
begin
  Result := DefaultInterface.Application;
end;

function TFolders.Get_Class_: OlObjectClass;
begin
  Result := DefaultInterface.Class_;
end;

function TFolders.Get_Session: _NameSpace;
begin
  Result := DefaultInterface.Session;
end;

function TFolders.Get_Parent: IDispatch;
begin
  Result := DefaultInterface.Parent;
end;

function TFolders.Get_Count: Integer;
begin
  Result := DefaultInterface.Count;
end;

function TFolders.Get_RawTable: IUnknown;
begin
  Result := DefaultInterface.RawTable;
end;

function TFolders.Item(Index: OleVariant): MAPIFolder;
begin
  Result := DefaultInterface.Item(Index);
end;

function TFolders.Add(const Name: WideString): MAPIFolder;
begin
  Result := DefaultInterface.Add(Name, EmptyParam);
end;

function TFolders.Add(const Name: WideString; Type_: OleVariant): MAPIFolder;
begin
  Result := DefaultInterface.Add(Name, Type_);
end;

function TFolders.GetFirst: MAPIFolder;
begin
  Result := DefaultInterface.GetFirst;
end;

function TFolders.GetLast: MAPIFolder;
begin
  Result := DefaultInterface.GetLast;
end;

function TFolders.GetNext: MAPIFolder;
begin
  Result := DefaultInterface.GetNext;
end;

function TFolders.GetPrevious: MAPIFolder;
begin
  Result := DefaultInterface.GetPrevious;
end;

procedure TFolders.Remove(Index: Integer);
begin
  DefaultInterface.Remove(Index);
end;
{$ENDIF VER130}

{ TMyFolders }

constructor TMyFolders.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Self.OnFolderAdd := DoFolderAdd;
  Self.OnFolderChange := DoFolderChange;
  Self.OnFolderRemove := DoFolderRemove;
end;

procedure TMyFolders.ConnectTo(AFolder: MAPIFolder);
begin
  if Assigned(AFolder) then
    inherited ConnectTo(AFolder.Folders);
end;

function TMyFolders.GetFolderObj: MAPIFolder;
begin
  Result := nil;
  if Assigned(DefaultInterface) then
    Result := DefaultInterface.Parent as MAPIFolder; 
end;

procedure TMyFolders.DoFolderAdd(ASender: TObject; const Folder: MAPIFolder);
begin
  ShowMessage('The "' + Folder.Name + '" subfolder is added to the "' + FolderObj.Name + '" folder.');
end;

procedure TMyFolders.DoFolderChange(ASender: TObject; const Folder: MAPIFolder);
begin
  ShowMessage('The "' + Folder.Name + '" subfolder is changed in the "' + FolderObj.Name + '" folder.');
end;

procedure TMyFolders.DoFolderRemove(Sender: TObject);
begin
  ShowMessage('A subfolder is removed from the "' + FolderObj.Name + '" folder.');
end;

{ TAddInModule }

procedure TAddInModule.adxCOMAddInModuleAddInStartupComplete(Sender: TObject);
begin
  FFolders := TMyFolders.Create(Self);
  if CmdBar.Controls[0].AsComboBox.ListIndex > 0 then
    ConnectToFolderByIndex(CmdBar.Controls[0].AsComboBox.ListIndex);
end;

procedure TAddInModule.adxCOMAddInModuleAddInBeginShutdown(Sender: TObject);
begin
  FFolders.Disconnect;
  FFolders := nil;
end;

procedure TAddInModule.CmdBarControls0Change(Sender: TObject);
begin
  if CmdBar.Controls[0].AsComboBox.ListIndex > 0 then
    ConnectToFolderByIndex(CmdBar.Controls[0].AsComboBox.ListIndex);
end;

procedure TAddInModule.CmdBarControls1Click(Sender: TObject);
begin
  if CmdBar.Controls[0].AsComboBox.ListIndex > 0 then
    ConnectToFolderByIndex(CmdBar.Controls[0].AsComboBox.ListIndex);
end;

procedure TAddInModule.ConnectToFolderByIndex(const AIndex: Integer);
var
  DefFolder: OlDefaultFolders;
begin
  case AIndex of
    1: DefFolder := olFolderDeletedItems;
    2: DefFolder := olFolderDrafts;
    3: DefFolder := olFolderInbox;
    4: DefFolder := olFolderOutbox;
    5: DefFolder := olFolderSentMail;
    6: DefFolder := olFolderContacts;
    7: DefFolder := olFolderTasks;
    8: DefFolder := olFolderCalendar;
  else
    DefFolder := olFolderInbox;
  end;
  FFolders.ConnectTo(OutlookApp.GetNamespace('MAPI').GetDefaultFolder(DefFolder));
end;

initialization
  TadxFactory.Create(ComServer, TADX_OLFoldersEvents, CLASS_ADX_OLFoldersEvents, TAddInModule);

end.
